@@ -1,11 +1,12 @@
-import email
 import json
 import logging
 import re
 import threading
 import time
+from email.parser import BytesParser
 from functools import partial
 from http import server, HTTPStatus
+from io import BytesIO
 from urllib.parse import parse_qs, urlparse, unquote
 
 from gcloud_storage_emulator import settings
@@ -33,7 +34,9 @@ def _health_check(req, res, storage):
 
 HANDLERS = (
     (r"^{}/b$".format(settings.API_ENDPOINT), {GET: buckets.ls, POST: buckets.insert}),
-    (r"^{}/b/(?P<bucket_name>[-.\w]+)$".format(settings.API_ENDPOINT), {GET: buckets.get, DELETE: buckets.delete}),
+    (
+        r"^{}/b/(?P<bucket_name>[-.\w]+)$".format(settings.API_ENDPOINT),
+        {GET: buckets.get, DELETE: buckets.delete}),
     (
         r"^{}/b/(?P<bucket_name>[-.\w]+)/o$".format(settings.API_ENDPOINT),
         {GET: objects.ls}
@@ -54,7 +57,9 @@ HANDLERS = (
         {POST: objects.insert, PUT: objects.upload_partial}
     ),
     (
-        r"^{}/b/(?P<bucket_name>[-.\w]+)/o/(?P<object_id>[-%.\w]+)$".format(settings.DOWNLOAD_API_ENDPOINT),
+        r"^{}/b/(?P<bucket_name>[-.\w]+)/o/(?P<object_id>(?:[-%.\w]+/)*[-%.\w]+)$".format(
+            settings.DOWNLOAD_API_ENDPOINT
+        ),
         {GET: objects.download},
     ),
 
@@ -63,7 +68,7 @@ HANDLERS = (
     (r"^/wipe$", {GET: _wipe_data}),  # Wipe all data
 
     # Public file serving, same as object.download
-    (r"^/(?P<bucket_name>[-.\w]+)/(?P<object_id>[-%.\w]+)$", {GET: objects.download}),
+    (r"^/(?P<bucket_name>[-.\w]+)/(?P<object_id>(?:[-%.\w]+/)*[-%.\w]+)$", {GET: objects.download}),
 )
 
 
@@ -80,9 +85,11 @@ def _read_data(request_handler):
         return json.loads(raw_data, encoding="utf-8")
 
     if content_type.startswith("multipart/"):
-        content = "Content-Type:" + content_type + "\r\n"
-        content += raw_data.decode("utf-8")
-        msg = email.message_from_string(content)
+        parser = BytesParser()
+        header = bytes("Content-Type:" + content_type + "\r\n", "utf-8")
+        file_ish = BytesIO(header + raw_data)
+        file_ish.seek(0)
+        msg = parser.parse(file_ish)
 
         payload = msg.get_payload()
 
@@ -90,7 +97,7 @@ def _read_data(request_handler):
         # object, and the second (and only other) part, the file content
         return {
             "meta": json.loads(payload[0].get_payload(), encoding="utf-8"),
-            "content": payload[1].get_payload(),
+            "content": payload[1].get_payload(decode=True),
             "content-type": payload[1].get_content_type(),
         }
 
@@ -216,7 +223,7 @@ class Router(object):
                 try:
                     handler(request, response, self._request_handler.storage)
                 except Exception as e:
-                    logger.error("An error has occured while running the handler for {} {}".format(
+                    logger.error("An error has occurred while running the handler for {} {}".format(
                         request.method,
                         request.full_url,
                     ))
